@@ -106,7 +106,7 @@ class SFTPServer(object):
                 os_flags |= os.O_TRUNC
             if flags & SSH2_FXF_EXCL and flags & SSH2_FXF_CREAT:
                 os_flags |= os.O_EXCL
-            mode = attrs.get('perm', 0o666)
+            mode = attrs.get(b'perm', 0o666)
             handle = self.storage.open(filename, os_flags, mode)
 
         if self.handle_cnt == 0xffffffffffffffff:
@@ -154,22 +154,23 @@ class SFTPServer(object):
         attrs = {}
         flags = self.consume_int()
         if flags & SSH2_FILEXFER_ATTR_SIZE:
-            attrs['size'] = self.consume_int64()
+            attrs[b'size'] = self.consume_int64()
         if flags & SSH2_FILEXFER_ATTR_UIDGID:
-            attrs['uid'] = self.consume_int()
-            attrs['gid'] = self.consume_int()
+            attrs[b'uid'] = self.consume_int()
+            attrs[b'gid'] = self.consume_int()
         if flags & SSH2_FILEXFER_ATTR_PERMISSIONS:
-            attrs['perm'] = self.consume_int()
+            attrs[b'perm'] = self.consume_int()
         if flags & SSH2_FILEXFER_ATTR_ACMODTIME:
-            attrs['atime'] = self.consume_int()
-            attrs['mtime'] = self.consume_int()
+            attrs[b'atime'] = self.consume_int()
+            attrs[b'mtime'] = self.consume_int()
         if flags & SSH2_FILEXFER_ATTR_EXTENDED:
             count = self.consume_int()
-            if count > 0:
-                attrs['extended'] = []
-            for i in range(0, count):
-                attrs['extended'].append(
-                    {self.consume_string(): self.consume_string()})
+            if count:
+                attrs[b'extended'] = [
+                    {self.consume_string(): self.consume_string()}
+                    for i in range(count)
+                ]
+
         return attrs
 
     def consume_filename(self, default=None):
@@ -191,12 +192,12 @@ class SFTPServer(object):
                 SSH2_FILEXFER_ATTR_PERMISSIONS | \
                 SSH2_FILEXFER_ATTR_ACMODTIME
         return struct.pack('>IQIIIII', flags,
-                           attrs['size'],
-                           attrs['uid'],
-                           attrs['gid'],
-                           attrs['perm'],
-                           int(attrs['atime']),
-                           int(attrs['mtime']))
+                           attrs[b'size'],
+                           attrs[b'uid'],
+                           attrs[b'gid'],
+                           attrs[b'perm'],
+                           int(attrs[b'atime']),
+                           int(attrs[b'mtime']))
 
     def send_msg(self, msg):
         msg_len = struct.pack('>I', len(msg))
@@ -275,17 +276,40 @@ class SFTPServer(object):
                 else:
                     self.send_status(msg_id, SSH2_FX_OP_UNSUPPORTED)
 
-    def send_item(self, sid, item, parent_dir=None, dummy=False):
-        msg = struct.pack('>BII', SSH2_FXP_NAME, sid, 1)
-        msg += struct.pack('>I', len(item)) + item
-        msg += struct.pack('>I', len(item)) + item
-        if not dummy:  # in case of a readlink response
-            if parent_dir:  # in case of readdir response
-                attrs = self.storage.stat(item, parent=parent_dir)
-            else:
-                attrs = self.storage.stat(item)
-            msg += self.encode_attrs(attrs)
+    def send_dummy_item(self, sid, item, filename):
+        # In case of readlink responses
+        # There's no need to add the attrs,
+        # But longname is still needed
+        # item is the linked and filename is the link
+        attrs = self.storage.stat(filename, lstat=True)
 
+        msg = struct.pack('>BII', SSH2_FXP_NAME, sid, 1)
+        msg += struct.pack('>I', len(item)) + item  # filename
+
+        if b'longname' in attrs and attrs[b'longname']:  # longname
+            longname = attrs[b'longname']
+        else:
+            longname = item
+        msg += struct.pack('>I', len(longname)) + longname
+
+        self.send_msg(msg)
+
+    def send_item(self, sid, item, parent_dir=None):
+        if parent_dir:  # in case of readdir response
+            attrs = self.storage.stat(item, parent=parent_dir)
+        else:
+            attrs = self.storage.stat(item)
+
+        msg = struct.pack('>BII', SSH2_FXP_NAME, sid, 1)
+        msg += struct.pack('>I', len(item)) + item  # filename
+
+        if b'longname' in attrs and attrs[b'longname']:  # longname
+            longname = attrs[b'longname']
+        else:
+            longname = item
+        msg += struct.pack('>I', len(longname)) + longname
+
+        msg += self.encode_attrs(attrs)
         self.send_msg(msg)
 
     def _realpath(self, sid):
@@ -391,7 +415,7 @@ class SFTPServer(object):
         attrs = self.consume_attrs()
         self.storage.mkdir(
             filename,
-            attrs.get('perm', 0o777)
+            attrs.get(b'perm', 0o777)
         )
         self.send_status(sid, SSH2_FX_OK)
 
@@ -420,7 +444,7 @@ class SFTPServer(object):
     def _readlink(self, sid):
         filename = self.consume_filename()
         link = self.storage.readlink(filename)
-        self.send_item(sid, link, dummy=True)
+        self.send_dummy_item(sid, link, filename)
 
     table = {
         SSH2_FXP_REALPATH: _realpath,
